@@ -12,7 +12,7 @@ import copy
 import logging
 import os
 from pathlib import Path
-
+import shutil
 
 import pandas as pd
 import requests
@@ -24,6 +24,7 @@ from metview.indexer import GribIndexer, FieldsetIndexer, ExperimentIndexer
 from metview.track import Track
 from metview.param import ParamInfo, ParamNameDesc, ParamIdDesc, init_pandas_options
 from metview import utils
+from metview.ipython import is_ipython_active
 
 
 ETC_PATH = os.path.join(os.path.dirname(__file__), "etc")
@@ -39,16 +40,17 @@ class IndexDb:
     def __init__(
         self,
         name,
-        label=None,
-        desc=None,
-        path=None,
-        rootdir_placeholder_value=None,
-        file_name_pattern=None,
-        db_dir=None,
+        label="",
+        desc="",
+        path="",
+        rootdir_placeholder_value="",
+        file_name_pattern="",
+        db_dir="",
         blocks=None,
         data_files=None,
         merge_conf=None,
         mapped_params=None,
+        regrid_from=None,
         dataset=None,
     ):
         self.name = name
@@ -56,19 +58,18 @@ class IndexDb:
         self.label = label
         if self.label is None or self.label == "":
             self.label = self.name
-        self.desc = "" if desc is None else desc
-        self.path = "" if path is None else path
+        self.desc = desc
+        self.path = path
 
-        self.rootdir_placeholder_value = (
-            "" if rootdir_placeholder_value is None else rootdir_placeholder_value
-        )
-        self.file_name_pattern = "" if file_name_pattern is None else file_name_pattern
+        self.rootdir_placeholder_value = rootdir_placeholder_value
+        self.file_name_pattern = file_name_pattern
         if self.file_name_pattern == "":
             self.path = os.path.dirname(self.path)
             self.file_name_pattern = os.path.basename(self.path)
 
-        self.db_dir = "" if db_dir is None else db_dir
+        self.db_dir = db_dir
         self.mapped_params = {} if mapped_params is None else mapped_params
+        self.regrid_from = [] if regrid_from is None else regrid_from
         self.blocks = {} if blocks is None else blocks
         self.vector_loaded = False
         self._param_types = {}
@@ -122,13 +123,10 @@ class IndexDb:
         # print(f"blocks={fs._db.blocks}")
         return fs
 
-    def _get_fields(self, dims, max_count=None):
+    def _get_fields(self, dims, max_count=-1):
         res = mv.Fieldset()
         dfs = {}
         LOG.debug(f"dims={dims}")
-
-        max_count = -1 if max_count is None else max_count
-        cnt = 0
         for key in self.blocks.keys():
             self._get_fields_for_block(key, dims, dfs, res, max_count)
             if max_count != -1 and len(res) >= max_count:
@@ -143,6 +141,7 @@ class IndexDb:
             blocks=dfs,
             label=self.label,
             mapped_params=self.mapped_params,
+            regrid_from=self.regrid_from,
         )
         return c, res
 
@@ -300,9 +299,11 @@ class IndexDb:
         p.load(self)
         return p
 
-    def describe(self, *args):
+    def describe(self, *args, **kwargs):
         param = args[0] if len(args) == 1 else None
-        return ParamNameDesc.describe(self, param=param)
+        if param is None:
+            param = kwargs.pop("param", None)
+        return ParamNameDesc.describe(self, param=param, **kwargs)
 
     def to_df(self):
         return pd.concat([p for _, p in self.blocks.items()])
@@ -381,10 +382,7 @@ class FieldsetDb(IndexDb):
         return df
 
     def _clone(self):
-        db = FieldsetDb(
-            self.name,
-            label=self.label,
-        )
+        db = FieldsetDb(self.name, label=self.label, regrid_from=self.regrid_from)
 
         if self._indexer is not None:
             db.indexer.update_keys(self._indexer.keys_ecc)
@@ -411,7 +409,7 @@ class FieldsetDb(IndexDb):
         # # return sorted(list(r))
         # return r
 
-    def ls(self, extra_keys=None, filter=None):
+    def ls(self, extra_keys=None, filter=None, no_print=False):
         default_keys = [
             "centre",
             "shortName",
@@ -460,16 +458,13 @@ class FieldsetDb(IndexDb):
                 df.drop("number", axis=1, inplace=True)
 
         init_pandas_options()
-        try:
-            import IPython
 
-            # test whether we're in the Jupyter environment
-            if IPython.get_ipython() is not None:
-                return df
-        except:
-            pass
-
-        print(df)
+        # test whether we're in the Jupyter environment
+        if is_ipython_active():
+            return df
+        elif not no_print:
+            print(df)
+        return df
 
     def speed(self):
         r = mv.Fieldset()
@@ -487,13 +482,13 @@ class FieldsetDb(IndexDb):
             r._db.load()
         return r
 
-    def deacc(self, skip_first=None):
+    def deacc(self, skip_first=False):
         if len(self.fs) > 1:
             self.load()
             step = self.unique("step")
             if step:
                 v = self.select(step=step[0])
-                if skip_first is None or skip_first == False:
+                if not skip_first:
                     r = v * 0
                 else:
                     r = mv.Fieldset()
@@ -537,8 +532,8 @@ class ExperimentDb(IndexDb):
         LOG.debug(f"rootdir_placeholder_value={self.rootdir_placeholder_value}")
 
     @staticmethod
-    def make_from_conf(name, conf, root_dir, db_root_dir, dataset):
-        LOG.debug(f"conf={conf}")
+    def make_from_conf(name, conf, root_dir, db_root_dir, regrid_conf, dataset):
+        # LOG.debug(f"conf={conf}")
         db = ExperimentDb(
             name,
             label=conf.get("label", ""),
@@ -554,6 +549,7 @@ class ExperimentDb(IndexDb):
             db_dir=os.path.join(db_root_dir, name),
             merge_conf=conf.get("merge", []),
             mapped_params={v: k for k, v in conf.get("mapped_params", {}).items()},
+            regrid_from=regrid_conf.get(name, []),
             blocks={},
             dataset=dataset,
         )
@@ -565,6 +561,7 @@ class ExperimentDb(IndexDb):
             label=self.label,
             db_dir=self.db_dir,
             mapped_params=self.mapped_params,
+            regrid_from=self.regrid_from,
             dataset=self.dataset,
             data_files=self.data_files,
             rootdir_placeholder_value=self.rootdir_placeholder_value,
@@ -749,12 +746,17 @@ class Dataset:
     )
     COMPRESSION = "bz2"
 
-    def __init__(self, name, path=None, load_style=True):
-        self.name = name
-        self.path = path if path is not None else ""
+    def __init__(self, name_or_path, load_style=True):
         self.field_conf = {}
         self.track_conf = {}
 
+        self.path = name_or_path
+        if any(x in self.path for x in ["/", "\\", "..", "./"]):
+            self.name = os.path.basename(self.path)
+        else:
+            self.name = self.path
+            self.path = ""
+
         assert self.name
         LOG.debug(f"name={self.name}")
 
@@ -803,13 +805,14 @@ class Dataset:
         data_conf_file = os.path.join(self.path, "data.yaml")
         with open(data_conf_file, "rt") as f:
             d = yaml.safe_load(f)
+            regrid_conf = d.get("regrid", {})
             for item in d["experiments"]:
                 ((name, conf),) = item.items()
                 if conf.get("type") == "track":
                     self.track_conf[name] = TrackConf(name, conf, data_dir, self)
                 else:
                     c = ExperimentDb.make_from_conf(
-                        name, conf, data_dir, index_dir, self
+                        name, conf, data_dir, index_dir, regrid_conf, self
                     )
                     self.field_conf[c.name] = c
 
@@ -904,3 +907,44 @@ class Dataset:
 
     def __str__(self):
         return f"{self.__class__.__name__}[name={self.name}]"
+
+
+def create_dataset_template(name_or_path):
+    path = name_or_path
+    if any(x in path for x in ["/", "\\", "..", "./"]):
+        name = os.path.basename(path)
+    else:
+        name = path
+        path = ""
+
+    if path == "":
+        path = os.path.join(Dataset.LOCAL_ROOT, name)
+
+    if not os.path.exists(path):
+        os.mkdir(path)
+    else:
+        if not os.path.isdir(path):
+            raise Exception(f"path must be a directory!")
+        if os.path.exists(os.path.join("path", "data.yaml")):
+            raise Exception(
+                f"The specified dataset directory={path} already exists and is not empty!"
+            )
+
+    # create dirs
+    for dir_name in ["conf", "data", "index"]:
+        os.mkdir(os.path.join(path, dir_name))
+
+    # copy files
+    files = {
+        "params.yaml": ("conf", ""),
+        "param_styles.yaml": ("conf", ""),
+        "areas.yaml": ("conf", ""),
+        "map_styles_template.yaml": ("conf", "map_styles.yaml"),
+        "dataset_template.yaml": ("", "data.yaml"),
+    }
+    for src_name, target in files.items():
+        target_dir = os.path.join(path, target[0]) if target[0] else path
+        target_name = target[1] if target[1] else src_name
+        shutil.copyfile(
+            os.path.join(ETC_PATH, src_name), os.path.join(target_dir, target_name)
+        )
