@@ -9,12 +9,12 @@
 #
 
 import logging
-from metview import dataset
 import re
 import pandas as pd
 
 import metview as mv
 from metview.indexer import GribIndexer
+from metview.ipython import is_ipython_active
 
 # logging.basicConfig(level=logging.INFO, format="%(levelname)s - %(message)s")
 # logging.basicConfig(level=logging.DEBUG, format="%(levelname)s - %(message)s")
@@ -312,17 +312,9 @@ class ParamDesc:
         return self._units
 
     @staticmethod
-    def describe(db, param=None):
-        in_jupyter = False
+    def describe(db, param=None, no_print=False):
         labels = {"marsClass": "class", "marsStream": "stream", "marsType": "type"}
-        try:
-            import IPython
-
-            # test whether we're in the Jupyter environment
-            if IPython.get_ipython() is not None:
-                in_jupyter = True
-        except:
-            pass
+        in_jupyter = is_ipython_active()
 
         # describe all the params
         if param is None:
@@ -373,7 +365,9 @@ class ParamDesc:
                 df = pd.DataFrame.from_dict(t)
                 df = df.set_index(["parameter"])
                 init_pandas_options()
-                print(df)
+                if not no_print:
+                    print(df)
+                return df
 
         # specific param
         else:
@@ -383,7 +377,7 @@ class ParamDesc:
             elif isinstance(param, int):
                 v = db.param_id_meta(param)
 
-            if v is None:
+            if v is None or len(v.md) == 0:
                 print(f"No shortName/paramId={param} found in data!")
                 return
 
@@ -430,11 +424,12 @@ class ParamDesc:
                 df = pd.DataFrame.from_dict(t)
                 df = df.set_index("key")
                 init_pandas_options()
-                print(df)
+                if not no_print:
+                    print(df)
+                return df
 
     @staticmethod
-    def _make_html_table(d, header=None):
-        header = header if header is not None else True
+    def _make_html_table(d, header=True):
         if len(d) > 1:
             first_column_name = list(d.keys())[0]
             txt = """  
@@ -464,7 +459,7 @@ class ParamDesc:
             return ""
 
     @staticmethod
-    def format_list(v, full=None):
+    def format_list(v, full=False):
         if isinstance(v, list):
             if full is True:
                 return ",".join([str(x) for x in v])
@@ -548,11 +543,11 @@ class ParamIdDesc(ParamDesc):
         self.md = {}
         self.levels = {}
 
-        # print(f"par={par}")
+        # print(f"par={par}"
         b_df = db.blocks.get("scalar", None)
         if b_df is not None:
-            q = f"paramId == '{self._param_id}'"
-            dft = b_df.query(q)
+            q = f"paramId == {self._param_id}"
+            dft = b_df.query(q, engine="python")
             if dft is not None:
                 for k in md.keys():
                     md[k].extend(dft[k].tolist())
