@@ -18,6 +18,7 @@ from pathlib import Path
 import yaml
 import metview as mv
 from metview.param import ParamInfo
+from metview.ipython import is_ipython_active
 
 # logging.basicConfig(level=logging.DEBUG)
 LOG = logging.getLogger(__name__)
@@ -208,14 +209,22 @@ class Style:
     def to_request(self):
         return [vd.to_request() for vd in self.visdefs]
 
-    def update(self, *args, inplace=None, verb=None):
+    def update(self, *args, inplace=False, verb=None):
         s = self if inplace == True else self.clone()
-        for i, v in enumerate(args):
-            if i < len(s.visdefs):
+        if isinstance(verb, str):
+            verb = [verb]
+        if verb:
+            for v in args:
                 if isinstance(v, dict):
-                    if verb is None or s.visdefs[i].verb in verb:
-                        v = {v_key.lower(): v_val for v_key, v_val in v.items()}
-                        s.visdefs[i].params.update(v)
+                    v_vals = {v_key.lower(): v_val for v_key, v_val in v.items()}
+                    for i, vd in enumerate(s.visdefs):
+                        if vd.verb in verb:
+                            vd.params.update(v_vals)
+        else:
+            for i, v in enumerate(args):
+                if isinstance(v, dict) and i < len(s.visdefs):
+                    v = {v_key.lower(): v_val for v_key, v_val in v.items()}
+                    s.visdefs[i].params.update(v)
         return s
 
     def set_data_id(self, data_id):
@@ -610,7 +619,10 @@ class MapConf:
             a = self.areas.get(area_v, {})
             if len(a) == 0 and area_v.upper() in self.BUILTIN_AREAS:
                 a = {"area_mode": "name", "area_name": area}
-        s = self.style_db.get_style(style_v)
+        if isinstance(style_v, mv.Request):
+            s = style_v
+        else:
+            s = self.style_db.get_style(style_v)
         return a, s
 
     def make_geo_view(self, area=None, style=None, plot_type=None):
@@ -618,22 +630,22 @@ class MapConf:
             style = "base_diff"
 
         a, s = self.find(area=area, style=style)
-        if s is not None:
-            s = s.clone()
-
-        if plot_type == "stamp":
-            s = s.update({"map_grid": "off", "map_label": "off"})
-
         if a is not None and a:
             a = copy.deepcopy(a)
         else:
             a = {}
 
-        if s is not None and s:
-            a["coastlines"] = s.to_request()
+        if s is not None:
+            if isinstance(s, mv.Request):
+                s = Visdef.from_request(s)
+            else:
+                s = s.clone()
+            if plot_type == "stamp":
+                s = s.update({"map_grid": "off", "map_label": "off"})
+            if s:
+                a["coastlines"] = s.to_request()
 
         return mv.geoview(**a)
-        # return GeoView(a, s)
 
 
 class StyleGallery:
@@ -646,11 +658,10 @@ class StyleGallery:
         with open(image_path, "rb") as img_file:
             return base64.b64encode(img_file.read()).decode("utf-8")
 
-    def build_gallery(self, names, images, row_height=None):
+    def build_gallery(self, names, images, row_height="150px"):
         figures = []
         # print(len(names))
         # print(len(images))
-        row_height = "150px" if row_height is None else row_height
         for name, image in zip(names, images):
             src = f"data:image/png;base64,{image}"
             caption = f'<figcaption style="text-align: center; font-size: 0.8em">{name}</figcaption>'
@@ -669,13 +680,7 @@ class StyleGallery:
         """
 
     def build(self):
-        try:
-            import IPython
-
-            # test whether we're in the Jupyter environment
-            if IPython.get_ipython() is None:
-                return
-        except:
+        if not is_ipython_active():
             return
 
         img_size = 120
@@ -700,8 +705,8 @@ class MapStyleGallery(StyleGallery):
         for name, d in map_styles().items():
             f_name = os.path.join(tmp_dir, name + ".png")
             if not os.path.exists(f_name):
-                view = map(area=[30, -30, 75, 45], style=name)
-                view.style.update({"MAP_COASTLINE_RESOLUTION": "low"}, inplace=True)
+                view = mv.make_geoview(area=[30, -30, 75, 45], style=name)
+                view.update({"MAP_COASTLINE_RESOLUTION": "low"}, sub="coastlines")
                 mv.setoutput(
                     mv.png_output(
                         output_name=f_name[:-4],
@@ -709,7 +714,7 @@ class MapStyleGallery(StyleGallery):
                         output_name_first_page_number="off",
                     )
                 )
-                mv.plot(view.to_request())
+                mv.plot(view)
             if os.path.exists(f_name):
                 names.append(name)
                 img.append(self.to_base64(f_name))
@@ -727,7 +732,7 @@ class MapAreaGallery(StyleGallery):
         for name in map_area_names():
             f_name = os.path.join(tmp_dir, name + ".png")
             if not os.path.exists(f_name):
-                view = map(area=name, style="grey_1")
+                view = mv.make_geoview(area=name, style="grey_1")
                 mv.setoutput(
                     mv.png_output(
                         output_name=f_name[:-4],
@@ -735,7 +740,7 @@ class MapAreaGallery(StyleGallery):
                         output_name_first_page_number="off",
                     )
                 )
-                mv.plot(view.to_request())
+                mv.plot(view)
             if os.path.exists(f_name):
                 names.append(name)
                 img.append(self.to_base64(f_name))
@@ -773,9 +778,12 @@ def make_geoview(**argv):
     return MAP_CONF().make_geo_view(**argv)
 
 
-def get_db(name=None):
+def make_eccharts_mcont():
+    return mv.mcont(contour_automatic_settings="ecmwf", legend="on")
+
+
+def get_db(name="param"):
     global _DB
-    name = "param" if name is None else name
     assert name in _DB
     if _DB[name][0] is None:
         _DB[name][0] = StyleDb(_DB[name][1], _DB[name][2])
@@ -791,10 +799,9 @@ def find(name):
         return None
 
 
-def load_custom_config(conf_dir, force=None):
+def load_custom_config(conf_dir, force=False):
     global CUSTOM_CONF_PATH
     global _MAP_CONF
-    force = False if force is None else force
     if CUSTOM_CONF_PATH:
         if CUSTOM_CONF_PATH[-1] == conf_dir:
             if force:
